"""
ansible-inspec: Compliance testing with Ansible and InSpec integration

Copyright (C) 2026 ansible-inspec project contributors
Licensed under GPL-3.0

This package combines Ansible and InSpec to provide infrastructure
compliance and security testing capabilities.
"""

__version__ = '0.2.7'
__author__ = 'Htunn Thu Thu'
__license__ = 'GPL-3.0'

# Upstream project information
UPSTREAM_PROJECTS = {
    'ansible': {
        'url': 'https://github.com/ansible/ansible',
        'license': 'GPL-3.0',
        'copyright': 'Red Hat, Inc.'
    },
    'inspec': {
        'url': 'https://github.com/inspec/inspec',
        'license': 'Apache-2.0',
        'copyright': 'Progress Software Corp.'
    }
}
